package top.daxian1218.dust.service.cache.redis;

import com.alibaba.fastjson.JSON;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.redis.core.RedisCallback;
import org.springframework.data.redis.core.StringRedisTemplate;
import top.daxian1218.dust.commons.util.TypeUtil;
import top.daxian1218.dust.db.cache.ICache;

import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;

public class RedisCache implements ICache {

    private final StringRedisTemplate stringRedisTemplate;

    public RedisCache(StringRedisTemplate stringRedisTemplate) {
        this.stringRedisTemplate = stringRedisTemplate;
    }

    @Override
    public void put(String key, Object value) {
        stringRedisTemplate.opsForValue().set(key, toRedisString(value));
    }

    private String toRedisString(Object value) {
        return TypeUtil.toStringOrJson(value);
    }

    @Override
    public void put(Map<String, Object> keyValues) {
        stringRedisTemplate.executePipelined((RedisCallback<Object>) connection -> {
            keyValues.forEach((k, v) -> {
                connection.set(k.getBytes(StandardCharsets.UTF_8), toRedisString(v).getBytes(StandardCharsets.UTF_8));
            });

            return null;
        });
    }

    @Override
    public Object get(String key) {
        if (StringUtils.isEmpty(key)) {
            return null;
        }

        return stringRedisTemplate.opsForValue().get(key);
    }

    @Override
    public boolean exists(String key) {
        return Boolean.TRUE.equals(stringRedisTemplate.hasKey(key));
    }

    @Override
    public void clear() {
        Set<String> keys = stringRedisTemplate.keys("*");
        if (keys != null) {
            stringRedisTemplate.delete(keys);
        }
    }

    @Override
    public List<String> keys() {
        Set<String> keys = stringRedisTemplate.keys("*");
        if (keys != null) {
            return new ArrayList<>(keys);
        }
        return new ArrayList<>();
    }

    @Override
    public void evict(String... keys) {
        if (keys != null && keys.length > 0) {
            stringRedisTemplate.delete(Arrays.asList(keys));
        }
    }
}
