package top.daxian1218.dust.service.config;

import com.alibaba.druid.support.http.StatViewServlet;
import com.alibaba.druid.support.http.WebStatFilter;
import org.springframework.boot.web.servlet.FilterRegistrationBean;
import org.springframework.boot.web.servlet.ServletRegistrationBean;
import top.daxian1218.dust.db.sql.pool.DruidTemplate;
import top.daxian1218.dust.db.sql.pool.DataSourceProperties;
import org.springframework.boot.autoconfigure.AutoConfigureAfter;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.util.Arrays;

/**
 * 引入druid连接池，需配置druid信息
 * @author hst
 */
@Configuration
@ConditionalOnProperty(value = "dust.db.pool-name", havingValue = "druid")
public class DruidAutoConfiguration {
    @Bean("druid")
    public DruidTemplate druidTemplate(DataSourceProperties dataSourceProperties) {
        return new DruidTemplate(dataSourceProperties);
    }

    @Bean
    @ConfigurationProperties("druid")
    public DataSourceProperties dataSourceProperties() {
        return new DataSourceProperties();
    }

    /**
     * 配置 Druid 的监控页
     * */
    @Bean
    public ServletRegistrationBean servletRegistrationBean(){

        StatViewServlet statViewServlet = new StatViewServlet();
        ServletRegistrationBean<StatViewServlet> statViewServletServletRegistrationBean = new ServletRegistrationBean<>(statViewServlet, "/druid/*");
        statViewServletServletRegistrationBean.addInitParameter("loginUsername","hezheng");  //给监控页设置 登录密码
        statViewServletServletRegistrationBean.addInitParameter("loginPassword","Hezheng@123");
        return statViewServletServletRegistrationBean;
    }

    /**
     *  WebStatFilter 用于采集 web -jdbc 关联监控的数据
     *
     * **/
    @Bean
    public FilterRegistrationBean webStatFilter(){
        WebStatFilter webStatFilter = new WebStatFilter();
        FilterRegistrationBean<WebStatFilter> webStatFilterFilterRegistrationBean =
                new FilterRegistrationBean<WebStatFilter>(webStatFilter);
        //采集哪个路径
        webStatFilterFilterRegistrationBean.setUrlPatterns(Arrays.asList("/*"));
        //排除那些路径 不采集
        //exclusions 你要排除 路径的 关键字
        webStatFilterFilterRegistrationBean.addInitParameter("exclusions","*.js,/druid/*");
        webStatFilterFilterRegistrationBean.setOrder(1);
        return webStatFilterFilterRegistrationBean;
    }
}
