package top.daxian1218.dust.service.config;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.event.ApplicationReadyEvent;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationListener;
import top.daxian1218.dust.commons.generic.Preconditions;
import top.daxian1218.dust.commons.id.SnowFlakeIdWorker;
import top.daxian1218.dust.commons.util.BeanUtil;
import top.daxian1218.dust.db.*;
import top.daxian1218.dust.db.base.DbTypes;
import top.daxian1218.dust.db.cache.CacheManager;
import top.daxian1218.dust.db.cache.SimpleMemoryCache;
import top.daxian1218.dust.db.dict.CacheDictObjContainer;
import top.daxian1218.dust.db.dict.DictObjContainer;
import top.daxian1218.dust.db.dict.IDictObjContainer;
import top.daxian1218.dust.db.tenant.AppManager;
import top.daxian1218.dust.db.util.DbUtil;

/**
 * @author hst
 */
public class DustApplicationListener implements ApplicationListener<ApplicationReadyEvent> {

    @Autowired(required = false)
    AppManager appManager;

    @Autowired
    DbManager dbManager;

    @Autowired
    DustDbProperties dustDbProperties;

    @Autowired
    DbAdapterManager dbAdapterManager;

    @Autowired
    CacheManager cacheManager;

    @Override
    public void onApplicationEvent(ApplicationReadyEvent event) {
        ApplicationContext ctx = event.getApplicationContext();
        if (ctx.containsBean("appManager")) {
            this.appManager = event.getApplicationContext().getBean(AppManager.class);
        }

        if (ctx.containsBean("dbManager")) {
            this.dbManager = event.getApplicationContext().getBean(DbManager.class);
        }
        if (ctx.containsBean("dustDbProperties")) {
            this.dustDbProperties = event.getApplicationContext().getBean(DustDbProperties.class);
        }
        if (ctx.containsBean("dbAdapterManager")) {
            this.dbAdapterManager = event.getApplicationContext().getBean(DbAdapterManager.class);
        }
        if (ctx.containsBean("cacheManager")) {
            this.cacheManager = event.getApplicationContext().getBean(CacheManager.class);
        }
        initSqlLog();
        initDb();
        initTenant();

    }

    private void initSqlLog() {
        DbUtil.getGlobalConfig().setWriteToDb(this.dustDbProperties.getLogSql().isEnable());
        DbUtil.getGlobalConfig().setDbAdapterManager(this.dbAdapterManager);
        DbUtil.getGlobalConfig().setDbName(this.dustDbProperties.getLogSql().getDbName());
    }

    public void initTenant() {
        if (appManager != null) {
            appManager.init();
        }
    }

    public void initDb() {
        Preconditions.checkNotNull(dbManager);

        DictGlobalConfig.Instance.setDefaultApp(dustDbProperties.getDict().getDefaultApp());
        DictGlobalConfig.Instance.setDictDb(dustDbProperties.getDict().getDbName());
        DictGlobalConfig.Instance.setDebug(dustDbProperties.getDict().isDebug());
        DictGlobalConfig.Instance.setNoCache(dustDbProperties.getDict().isNoCache());
        DictGlobalConfig.Instance.setCacheManager(cacheManager);

        DictGlobalConfig.Instance.setObjContainer(getContainer(dustDbProperties.getDict().getContainerClass(), dbAdapterManager));

        if (StringUtils.isNotEmpty(dustDbProperties.getDateTimeFormatter())) {
            DictGlobalConfig.Instance.setDateFormat(dustDbProperties.getDateTimeFormatter());
        }

        if (dustDbProperties.getWorkId() != null && dustDbProperties.getDatacenterId() != null) {
            DictGlobalConfig.Instance.setIdWorker(new SnowFlakeIdWorker(dustDbProperties.getWorkId(), dustDbProperties.getDatacenterId()));
        }

        if (!cacheManager.getCacheNames().contains(DbConstants.CACHE_DICT)) {
            cacheManager.register(DbConstants.CACHE_DICT, new SimpleMemoryCache());
        }

        dbManager.startSync();
    }

    private IDictObjContainer getDefaultContainer(DbAdapterManager dbAdapterManager) {
        return DictGlobalConfig.Instance.isNoCache() ? new DictObjContainer(dbAdapterManager) :
                new CacheDictObjContainer(dbAdapterManager);
    }

    private IDictObjContainer getContainer(String containerClass, DbAdapterManager dbAdapterManager) {
        if (StringUtils.isEmpty(containerClass)) {
            return getDefaultContainer(dbAdapterManager);
        }

        switch (containerClass) {
            case DbTypes.Mssql:
            case DbTypes.Mysql:
            case DbTypes.Oracle:
            case DbTypes.Sqlite:
            case DbTypes.Highgo:
                return getDefaultContainer(dbAdapterManager);
            default:
                return (IDictObjContainer) BeanUtil.getBean(containerClass);
        }
    }
}
