package top.daxian1218.dust.service.config;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.AutoConfigureAfter;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import top.daxian1218.dust.db.DbAdapterManager;
import top.daxian1218.dust.db.DbManager;
import top.daxian1218.dust.db.DustDbProperties;
import top.daxian1218.dust.db.cache.CacheManager;
import top.daxian1218.dust.db.cache.ICacheManager;
import top.daxian1218.dust.db.sql.pool.IDataSourceTemplate;

/**
 * DustDB公共配置类
 *
 * @author hst
 */
@Configuration
@AutoConfigureAfter({CommonsAutoConfiguration.class})
public class DustDbAutoConfiguration {

    @Autowired
    IDataSourceTemplate dataSourceTemplate;

    @Bean(initMethod = "loadConfig")
    public DbManager dbManager() {
        DustDbProperties dbProperties = dustDbProperties();
        DbManager dbManager = new DbManager(dataSourceTemplate);
        dbManager.setRemoteConfigDb(dbProperties.getDbName());
        dbProperties.getDbList().forEach(dbAccess -> {
            dbManager.getLocalDbList().add(dbAccess.clone());
        });

        return dbManager;
    }

    @Bean
    public DbAdapterManager dbAdapterManager() {
        return new DbAdapterManager(dbManager());
    }

    @Bean
    @ConfigurationProperties("dust.db")
    public DustDbProperties dustDbProperties() {
        return new DustDbProperties();
    }

    @Bean
    public ICacheManager cacheManager() {
        return new CacheManager();
    }

}
