package top.daxian1218.dust.service.config;

import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.connection.RedisStandaloneConfiguration;
import org.springframework.data.redis.connection.lettuce.LettuceConnectionFactory;
import org.springframework.data.redis.core.StringRedisTemplate;
import top.daxian1218.dust.db.DbConstants;
import top.daxian1218.dust.db.cache.CacheManager;
import top.daxian1218.dust.service.cache.redis.RedisCache;

@Configuration
@ConditionalOnProperty("spring.redis.host")
public class RedisConfiguration {

    final String REDIS_CACHE = "redis";

    private final CacheManager cacheManager;

    public RedisConfiguration(CacheManager cacheManager) {
        this.cacheManager = cacheManager;
    }

    @Bean("redisCache")
    public RedisCache redisCache(@Qualifier("dictRedisTemplate")  StringRedisTemplate stringRedisTemplate, @Qualifier("stringRedisTemplate") StringRedisTemplate stringRedisTemplate1) {
        RedisCache cache = new RedisCache(stringRedisTemplate);
        this.cacheManager.register(DbConstants.CACHE_DICT, cache);
        this.cacheManager.register(REDIS_CACHE, new RedisCache(stringRedisTemplate1));
        return cache;
    }

    @Bean("dictRedisTemplate")
    public StringRedisTemplate dictRedisTemplate(RedisConnectionFactory redisConnectionFactory) {
        LettuceConnectionFactory defaultFactory = (LettuceConnectionFactory) redisConnectionFactory;
        StringRedisTemplate template = new StringRedisTemplate();
        RedisStandaloneConfiguration config = new RedisStandaloneConfiguration(defaultFactory.getHostName(), defaultFactory.getPort());
        config.setPassword(defaultFactory.getPassword());
        config.setDatabase(15);
        LettuceConnectionFactory factory = new LettuceConnectionFactory(config);
        factory.afterPropertiesSet();
        template.setConnectionFactory(factory);
        return template;
    }

}
