package top.daxian1218.dust.service.config;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.boot.autoconfigure.AutoConfigureAfter;
import org.springframework.boot.autoconfigure.AutoConfigureBefore;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.autoconfigure.security.servlet.SecurityAutoConfiguration;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpMethod;
import org.springframework.security.config.annotation.authentication.builders.AuthenticationManagerBuilder;
import org.springframework.security.config.annotation.method.configuration.EnableGlobalMethodSecurity;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.builders.WebSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.annotation.web.configuration.WebSecurityConfigurerAdapter;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;
import top.daxian1218.dust.service.security.AuthoritiesConstants;
import top.daxian1218.dust.service.security.JwtAuthorizationFilter;
import top.daxian1218.dust.service.security.jwt.SignProvider;
import top.daxian1218.dust.service.security.jwt.TokenProvider;

/**
 * @author hst
 */
@Configuration
public class SecurityConfiguration {

    @Configuration
    @EnableConfigurationProperties(DustMsProperties.class)
    @EnableWebSecurity
    static class DustSecurityConfiguration extends WebSecurityConfigurerAdapter {

        public DustSecurityConfiguration(DustMsProperties dustMsProperties) {
            this.dustMsProperties = dustMsProperties;
        }

        private DustMsProperties dustMsProperties;

        @Bean
        public TokenProvider tokenProvider() {
            return new TokenProvider(this.dustMsProperties);
        }

        @Bean
        public SignProvider signProvider() {
            return new SignProvider();
        }

        /**
         * 配置请求忽略列表
         * @param web
         * @throws Exception
         */
        @Override
        public void configure(WebSecurity web) throws Exception {
            web.ignoring()
                    .antMatchers(HttpMethod.OPTIONS, "/**")
                    .antMatchers("/*.ico")
                    .antMatchers("/druid/**")
                    .antMatchers("/app/**/*.{js,html,css,ico,png,jpg}")
                    .antMatchers("/bower_components/**")
                    .antMatchers("/i18n/**")
                    .antMatchers("/content/**")
                    .antMatchers("/swagger-ui/index.html")
                    .antMatchers("/test/**")
                    .antMatchers("/h2-console/**");

            String[] matchers = dustMsProperties.getSecurity().getIgnoreMatchers();
            if (matchers != null) {
                web.ignoring().antMatchers(matchers);
            }
        }

        /**
         * 配置相关服务的安全性以及过滤器
         * @param http
         * @throws Exception
         */
        @Override
        protected void configure(HttpSecurity http) throws Exception {
            http
                    .cors()
                    .and()
                    .csrf().disable()
                    .headers().frameOptions().disable()
                    .and()
                    .sessionManagement().sessionCreationPolicy(SessionCreationPolicy.STATELESS)
                    .and()
                    .authorizeRequests()
                    .antMatchers("/api/**").authenticated()
                    .antMatchers("/management/**").hasAuthority(AuthoritiesConstants.ADMIN)
                    .antMatchers("/swagger-resources/configuration/ui").permitAll()
                    .anyRequest().permitAll()
                    .and()
                    .addFilterBefore(new JwtAuthorizationFilter(tokenProvider(), signProvider()), UsernamePasswordAuthenticationFilter.class)
                    .httpBasic().disable();
        }

    }
}


