package top.daxian1218.dust.service.config;

import com.alibaba.fastjson.serializer.SerializerFeature;
import com.alibaba.fastjson.support.spring.FastJsonHttpMessageConverter;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.boot.web.servlet.FilterRegistrationBean;
import org.springframework.boot.web.servlet.ServletContextInitializer;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.Ordered;
import org.springframework.http.MediaType;
import org.springframework.http.converter.HttpMessageConverter;
import org.springframework.web.cors.CorsConfiguration;
import org.springframework.web.cors.UrlBasedCorsConfigurationSource;
import org.springframework.web.filter.CorsFilter;
import org.springframework.web.servlet.config.annotation.WebMvcConfigurer;
import top.daxian1218.dust.commons.data.HDataRow;
import top.daxian1218.dust.commons.data.HDataRowCollection;
import top.daxian1218.dust.commons.data.HDataTable;
import top.daxian1218.dust.commons.json.HDataRowCollectionSerializer;
import top.daxian1218.dust.commons.json.HDataRowSerializer;
import top.daxian1218.dust.commons.json.HDataTableSerializer;
import top.daxian1218.dust.db.tenant.TenantManager;
import top.daxian1218.dust.service.common.DustJsonValueFilter;
import top.daxian1218.dust.service.web.ResponseHandler;
import top.daxian1218.dust.service.web.filter.HttpHeadersFilter;
import top.daxian1218.dust.service.web.filter.TenantFilter;

import javax.servlet.ServletContext;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.List;

/**
 * 配置Web应用所需的API信息
 *
 * @author hst
 */
@Configuration
@EnableConfigurationProperties(DustMsProperties.class)
public class WebConfiguration
        implements WebMvcConfigurer, ServletContextInitializer {

    private final Logger log = LoggerFactory.getLogger(WebConfiguration.class);

    DustMsProperties dustMsProperties;

    public WebConfiguration(DustMsProperties dustMsProperties) {
        this.dustMsProperties = dustMsProperties;
    }


    @Override
    public void onStartup(ServletContext servletContext) {

    }

    @Bean
    public ResponseHandler exceptionsHandler() {
        return new ResponseHandler();
    }

    @Bean
    public FilterRegistrationBean<HttpHeadersFilter> sysParamFilter() {
        FilterRegistrationBean<HttpHeadersFilter> filterRegistrationBean =
                new FilterRegistrationBean<>(new HttpHeadersFilter());
        filterRegistrationBean.setOrder(Ordered.HIGHEST_PRECEDENCE - 2);
        return filterRegistrationBean;
    }

    @Bean
    public FilterRegistrationBean<TenantFilter> repositoryFilter(TenantManager tenantManager) {
        FilterRegistrationBean<TenantFilter> filterRegistrationBean =
                new FilterRegistrationBean<>(new TenantFilter(dustMsProperties, tenantManager));
        filterRegistrationBean.setOrder(Ordered.HIGHEST_PRECEDENCE - 1);
        return filterRegistrationBean;
    }

    @Bean
    @ConditionalOnProperty(name = "dust.ms.cors.allowed-origins")
    public CorsFilter corsFilter() {
        log.debug("Registering CORS filter");
        UrlBasedCorsConfigurationSource source = new UrlBasedCorsConfigurationSource();
        CorsConfiguration config = dustMsProperties.getCors();
        source.registerCorsConfiguration("/api/**", config);
        source.registerCorsConfiguration("/v2/api-docs", config);
        source.registerCorsConfiguration("/oauth/**", config);
        String[] matchers = dustMsProperties.getCorsMatchers();
        if (matchers != null) {
            for (int i = 0; i < matchers.length; i++) {
                source.registerCorsConfiguration(matchers[i], config);
            }
        }
        return new CorsFilter(source);
    }

    @Override
    public void configureMessageConverters(List<HttpMessageConverter<?>> converters) {

        FastJsonHttpMessageConverter fastJsonHttpMessageConverter = new FastJsonHttpMessageConverter();
        List<MediaType> mediaTypes = new ArrayList<>();
        mediaTypes.add(MediaType.ALL);
        fastJsonHttpMessageConverter.setSupportedMediaTypes(mediaTypes);
        fastJsonHttpMessageConverter.getFastJsonConfig().setSerializerFeatures(
                SerializerFeature.WriteMapNullValue,
                SerializerFeature.WriteNullBooleanAsFalse,
                SerializerFeature.WriteEnumUsingToString);

        fastJsonHttpMessageConverter.getFastJsonConfig().getSerializeConfig().put(HDataTable.class, new HDataTableSerializer());
        fastJsonHttpMessageConverter.getFastJsonConfig().getSerializeConfig().put(HDataRow.class, new HDataRowSerializer());
        fastJsonHttpMessageConverter.getFastJsonConfig().getSerializeConfig().put(HDataRowCollection.class, new HDataRowCollectionSerializer());

        fastJsonHttpMessageConverter.setDefaultCharset(StandardCharsets.UTF_8);
        fastJsonHttpMessageConverter.getFastJsonConfig().setSerializeFilters(new DustJsonValueFilter());
        converters.add(0, fastJsonHttpMessageConverter);

    }
}
