package top.daxian1218.dust.service.security;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.springframework.security.authentication.AbstractAuthenticationToken;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.util.Assert;
import org.springframework.util.StringUtils;
import top.daxian1218.dust.commons.generic.Preconditions;

import java.security.Principal;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * 微服务的认证信息
 *
 * @author hst
 */
public class DustAuthentication extends AbstractAuthenticationToken {

    private String credential;
    private Principal principal;
    private DustUser dustUser;

    public DustAuthentication(DustUser user) {
        this(user.getAuthorities());
        this.principal = new DustPrincipal(user.getId());
        this.setDetails(user);

        dustUser = user;
        this.credential = user.getToken();
        setAuthenticated(true);
    }

    private DustAuthentication(Collection<? extends GrantedAuthority> authorities) {
        super(authorities);
    }

    @Override
    public Object getCredentials() {
        return credential;
    }

    @Override
    public Object getPrincipal() {
        return principal;
    }

    public DustUser getUser() {
        return dustUser;
    }

    @Override
    public Object getDetails() {
        return super.getDetails();
    }

    public class DustPrincipal implements Principal {
        private final String name;

        public DustPrincipal(String name) {
            this.name = name;
        }

        @Override
        public String getName() {
            return this.name;
        }
    }
}
