package top.daxian1218.dust.service.security;

import org.apache.commons.lang3.StringUtils;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import top.daxian1218.dust.commons.generic.Preconditions;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 系统参数类
 * 时间戳(timestamp)、应用ID(appId)、签名(sign)、租户(tenantId)
 * 签名在安全处验证，该处只做标识
 *
 * @author hst
 */
public class DustUser {
    /**
     * system parameter
     */
    private Long timestamp;
    private String appId;
    private String tenantId;

    /**
     * user info
     */

    private String sid;
    private String id;
    private String code;
    private String name;
    private String departmentId;
    private String orgId;
    private String orgCode;

    /**
     * user right
     */
    private String roles;
    private String rights;
    private String token;

    private String refreshToken;

    private Map<String, String> propsMap = new HashMap<>();

    public DustUser() {

    }

    public DustUser(String appId, String tenantId, Long timestamp) {
        this.appId = appId;
        this.tenantId = tenantId;
        this.timestamp = timestamp;
    }

    public long getTimestamp() {
        return timestamp;
    }

    public void setTimestamp(long timestamp) {
        this.timestamp = timestamp;
    }

    public String getAppId() {
        return appId;
    }

    public void setAppId(String appId) {
        this.appId = appId;
    }

    public String getTenantId() {
        return tenantId;
    }

    public void setTenantId(String tenantId) {
        this.tenantId = tenantId;
    }

    public void setTimestamp(Long timestamp) {
        this.timestamp = timestamp;
    }

    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }

    public String getCode() {
        return code;
    }

    public void setCode(String code) {
        this.code = code;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getDepartmentId() {
        return departmentId;
    }

    public void setDepartmentId(String departmentId) {
        this.departmentId = departmentId;
    }

    public String getOrgId() {
        return orgId;
    }

    public void setOrgId(String orgId) {
        this.orgId = orgId;
    }

    public String getRoles() {
        return roles;
    }

    public void setRoles(String roles) {
        this.roles = roles;
    }

    public String getRights() {
        return rights;
    }

    public void setRights(String rights) {
        this.rights = rights;
    }

    public String getToken() {
        return token;
    }

    public void setToken(String token) {
        this.token = token;
    }

    public String getSid() {
        return sid;
    }

    public void setSid(String sid) {
        this.sid = sid;
    }

    public String getRefreshToken() {
        return refreshToken;
    }

    public void setRefreshToken(String refreshToken) {
        this.refreshToken = refreshToken;
    }

    public String getOrgCode() {
        return orgCode;
    }

    public void setOrgCode(String orgCode) {
        this.orgCode = orgCode;
    }

    public String getProps() {
        return propsMap.entrySet().stream().map(entry -> {
            return entry.getKey() + "=" + entry.getValue();
        }).collect(Collectors.joining("&"));
    }

    public void setProps(String props) {
        this.propsMap.clear();
        if (StringUtils.isNotEmpty(props)) {
            String[] arr = props.split("[&=]");
            for (int i = 0; i < arr.length - 1;i += 2) {
                propsMap.put(arr[i].toUpperCase(), arr[i + 1]);
            }
        }
    }

    private String GetProp(String key)
    {
        if (propsMap.containsKey(key.toUpperCase()))
        {
            return propsMap.get(key.toUpperCase());
        }

        return StringUtils.EMPTY;
    }

    public String get(String key) {
        Preconditions.checkNotNull(key);

        key = key.toUpperCase();
        switch (key) {
            case "ID":
                return id;
            case "CODE":
                return code;
            case "NAME":
                return name;
            case "DEPARTMENTID":
                return departmentId;
            case "ORGID":
                return orgId;
            case "ROLES":
                return roles;
            case "TENANTID":
                return tenantId;
            case "APPID":
                return appId;
            case "RIGHTS":
                return rights;
            default:
                return GetProp(key);
        }
    }

    public  DustUser set(String key, String value) {
        Preconditions.checkNotEmpty(key, "key");
        switch (key.toUpperCase())
        {
            case "ID":
                id = value;
                break;
            case "CODE":
                code = value;
                break;
            case "NAME":
                name = value;
                break;
            case "DEPARTMENTID":
                departmentId = value;
                break;
            case "ORGID":
                orgId = value;
                break;
            case "ROLES":
                roles = value;
                break;
            case "TENANTID":
                tenantId = value;
                break;
            case "APPID":
                appId = value;
                break;
            case "RIGHTS":
                rights = value;
                break;
            default:
                propsMap.put(key, value);
                break;
        }

        return this;
    }

    public Collection<SimpleGrantedAuthority> getAuthorities() {
        return Arrays.stream(Optional.ofNullable(roles)
                .orElse(StringUtils.EMPTY).split(","))
                .filter(StringUtils::isNotEmpty)
                .map(SimpleGrantedAuthority::new).collect(Collectors.toList());
    }


    public Map<String, Object> toClaims() {
        Map<String, Object> claims = new HashMap<>();
        claims.put("sid", sid);
        claims.put("id", id);
        claims.put("code", code);
        claims.put("name", name);
        claims.put("orgId", orgId);
        claims.put("orgCode", orgCode);
        claims.put("roles", roles);

        claims.put("tenantId", tenantId);
        claims.put("appId", appId);

        if (!StringUtils.isEmpty(rights)) {
            claims.put("rights", rights);
        }

        claims.put("props", getProps());
        return claims;

    }
}
