package top.daxian1218.dust.service.security;

import org.springframework.http.MediaType;
import org.springframework.web.filter.OncePerRequestFilter;
import top.daxian1218.dust.service.security.jwt.SignProvider;
import top.daxian1218.dust.service.security.jwt.TokenProvider;
import top.daxian1218.dust.service.util.ResponseUtil;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.nio.charset.StandardCharsets;

/**
 * jwt授权
 * @author hst
 */
public class JwtAuthorizationFilter extends OncePerRequestFilter {

    private TokenProvider tokenProvider;

    private SignProvider signProvider;

    public JwtAuthorizationFilter(TokenProvider tokenProvider, SignProvider signProvider) {
        super();
        this.tokenProvider = tokenProvider;
        this.signProvider = signProvider;
    }

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) throws ServletException, IOException {

        //验证数据安全性
        if (!this.signProvider.validateRequest(request)) {
            debuggerLog("数据加密认证失败，返回400错误");
            response.setStatus(HttpServletResponse.SC_BAD_REQUEST);
            return;
        }

        //验证JWT,用户信息
        if (!this.tokenProvider.validateRequest(request)) {
            debuggerLog("授权认证失败，返回401错误");
            response.setStatus(HttpServletResponse.SC_UNAUTHORIZED);
            response.setContentType(MediaType.APPLICATION_JSON_VALUE);
            response.setCharacterEncoding(StandardCharsets.UTF_8.displayName());
            response.getWriter().print(ResponseUtil.authorizeFailure());
            return;
        }

        filterChain.doFilter(request, response);
    }

    private void debuggerLog(String msg) {
        if (logger.isDebugEnabled()) {
            logger.debug(msg);
        }
    }
}
