package top.daxian1218.dust.service.sql;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.config.YamlPropertiesFactoryBean;
import org.springframework.core.env.PropertiesPropertySource;
import org.springframework.core.env.PropertySource;
import org.springframework.core.io.support.EncodedResource;
import org.springframework.core.io.support.PropertySourceFactory;
import top.daxian1218.dust.db.sql.pool.SqlResource;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

/**
 * Sql配置读取Factory
 * 配置不会再SpringBoot生效，恒返回空配置
 */
public class SqlYamlPropertySourceFactory implements PropertySourceFactory {
    @Override
    public PropertySource<?> createPropertySource(String name, EncodedResource resource) throws IOException {
        // 请配置注解参数：ignoreResourceNotFound = true
        try {
            if (StringUtils.isEmpty(resource.getResource().getFilename())) {
                throw new FileNotFoundException("no file name");
            }
            String filename = resource.getResource().getFilename();
            if (StringUtils.isEmpty(name)) {
                name = filename.replace(".yml", StringUtils.EMPTY);
            }
            //1.创建yaml文件解析工厂
            YamlPropertiesFactoryBean yaml = new YamlPropertiesFactoryBean();
            //2.设置资源内容
            yaml.setResources(resource.getResource());
            //3.解析成properties文件
            Properties properties = yaml.getObject();
            if (properties == null) {
                throw new FileNotFoundException();
            }

            Map<String, String> mapSql = new HashMap<>();
            for (String propName : properties.stringPropertyNames()) {
                mapSql.put(propName, properties.getProperty(propName));
            }

            SqlResource.addDbSqlRes(name, mapSql);

            //4.返回符合spring的PropertySource对象
            return new PropertiesPropertySource(name, new Properties());
        } catch (Exception e) {
            e.printStackTrace();
            throw new IOException("Yaml解析异常，异常原因：" + e.getMessage());
        }
    }
}
