package top.daxian1218.dust.service.util;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import top.daxian1218.dust.commons.id.SnowFlakeIdWorker;
import top.daxian1218.dust.commons.util.BeanUtil;
import top.daxian1218.dust.db.DbAdapterManager;
import top.daxian1218.dust.db.DustDbException;
import top.daxian1218.dust.db.base.IDataAdapter;
import top.daxian1218.dust.db.sql.HSqlCommand;
import top.daxian1218.dust.service.config.DustMsProperties;

import java.util.Date;

/**
 * 用于记录日志使用
 *
 * @author hst
 */
public class LogUtil {

    private final static Logger logger = LoggerFactory.getLogger(LogUtil.class);

    private static final String SQL_LOG_INSERT = "INSERT INTO dust_log(id, log_time, content) VALUES(:id, :logTime, :content)";

    private static DustMsProperties dustMsProperties;
    private static DbAdapterManager dbAdapterManager;

    private static void init() {
        if (dbAdapterManager == null) {
            dbAdapterManager = BeanUtil.getBean(DbAdapterManager.class);
        }
        if (dustMsProperties == null) {
            dustMsProperties = BeanUtil.getBean(DustMsProperties.class);
        }
    }


    public static void write(String message, Throwable e) {
        init();

        if (!dustMsProperties.getLogging().getLogdb().isEnabled()) {
            logger.debug(message, e);
            return;
        }

        IDataAdapter adapter = null;
        try {
            adapter = dbAdapterManager.getAdapter(dustMsProperties.getLogging().getLogdb().getDataSource());
        } catch (DustDbException dustDbException) {
            logger.error("记录日志，数据库无法访问。错误信息：" + message, e);
        }

        HSqlCommand cmd = new HSqlCommand(SQL_LOG_INSERT);
        try {
            String str = e != null ? String.format("%s:%s", message, e.getStackTrace()) : message;
            cmd.setParameter("content", str);
            cmd.setParameter("id", SnowFlakeIdWorker.getInstance0().nextId());
            cmd.setParameter("logTime", new Date());
            adapter.update(cmd);
            adapter.commit();
            adapter.closeQuiet();
        } catch (DustDbException ex) {
            adapter.closeQuiet();
            logger.error("记录日志发生异常，发生详情： {}.{} with cause = {}", LogUtil.class, "write", ex.getCause());
        }
    }

    public static void write(Throwable e) {
        write(e.getMessage(), e);
    }

    public static void write(String message) {
        write(message, null);
    }
}
