package top.daxian1218.dust.service.util;

import com.alibaba.fastjson.JSONObject;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import top.daxian1218.dust.commons.exception.DustException;
import top.daxian1218.dust.commons.thread.LocalHolder;
import top.daxian1218.dust.service.common.DustResponse;
import top.daxian1218.dust.service.security.jwt.IAuthentication;

import java.util.Arrays;

/**
 * @author hst
 */
public class ResponseUtil {

    static Logger logger = LoggerFactory.getLogger(ResponseUtil.class);

    public static DustResponse success(Object data) {
        DustResponse response = new DustResponse();
        response.setStatus(200);
        response.setData(data);
        return response;
    }

    public static DustResponse exception(DustException ex) {
        logger.error(ex.getMessage(), ex);
        DustResponse response = new DustResponse();
        response.setStatus(ex.getStatus());
        response.setError(ex.getError());
        response.setMessage(ex.getMessage());
        if (ex.getCause() != null) {
            response.setException(ex.getCause().getMessage());
        }
        return response;
    }

    public static DustResponse exception(Exception ex) {
        logger.error(ex.getMessage(), ex);
        Throwable throwable = ex;
        if (ex.getCause() != null) {
            throwable = ex.getCause();
        }

        DustResponse response = new DustResponse();
        response.setStatus(500);
        response.setError("S0001");
        response.setMessage(StringUtils.isEmpty(throwable.getMessage())
                ? throwable.toString()
                : throwable.getMessage());
        return response;
    }

    public static String authorizeFailure() {
        String message = LocalHolder.get(IAuthentication.AUTHORIZE_ERROR);
        if (StringUtils.isEmpty(message)) {
            message = "授权校验失败";
        }

        logger.error(message);

        JSONObject result = new JSONObject();
        result.put("status", 401);
        result.put("error", "S0001");
        result.put("message", message);
        return result.toJSONString();
    }
}
