package top.daxian1218.dust.service.web;

import com.alibaba.fastjson.JSONObject;
import org.springframework.core.MethodParameter;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.servlet.mvc.method.annotation.ResponseBodyAdvice;
import top.daxian1218.dust.commons.exception.DustException;
import top.daxian1218.dust.service.annotation.CustomMapping;
import top.daxian1218.dust.service.common.DustResponse;
import top.daxian1218.dust.service.util.ResponseUtil;

/**
 * @author daxian
 */
@RestControllerAdvice
public class ResponseHandler implements ResponseBodyAdvice {



    /**
     * 规定错误
     */
    @ExceptionHandler({DustException.class})
    public DustResponse exception(DustException ex) {
        return ResponseUtil.exception(ex);
    }

    /**
     * 其他错误
     */
    @ExceptionHandler({Exception.class})
    public DustResponse exception(Exception ex) {
        return ResponseUtil.exception(ex);
    }

    @Override
    public boolean supports(MethodParameter returnType, Class converterType) {
        if (returnType.getMethod().isAnnotationPresent(CustomMapping.class)) {
            return false;
        }
        return true;
    }

    @Override
    public Object beforeBodyWrite(Object body, MethodParameter returnType, MediaType selectedContentType, Class selectedConverterType, ServerHttpRequest request, ServerHttpResponse response) {
        if (body instanceof DustResponse) {
            if (((DustResponse) body).getStatus() == 401) {
                response.setStatusCode(HttpStatus.UNAUTHORIZED);
            }
            return body;
        }

        return ResponseUtil.success(body);
    }
}
