package top.daxian1218.dust.service.web.filter;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.filter.GenericFilterBean;
import top.daxian1218.dust.commons.exception.DustSystemException;
import top.daxian1218.dust.commons.thread.LocalHolder;
import top.daxian1218.dust.db.DbAdapterManager;
import top.daxian1218.dust.db.DictGlobalConfig;
import top.daxian1218.dust.db.base.IDataAdapter;
import top.daxian1218.dust.db.tenant.TenantManager;
import top.daxian1218.dust.service.config.DustMsProperties;
import top.daxian1218.dust.service.security.DustAuthentication;
import top.daxian1218.dust.service.security.DustUser;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.util.regex.Pattern;

/**
 * 数据仓库过滤器
 *
 * @author hst
 */
public class TenantFilter extends GenericFilterBean {
    static Pattern IGNORE_URI = Pattern.compile( "\\.\\w+$|/druid.*");
    DustMsProperties dustMsProperties;
    TenantManager tenantAdapterManager;


    public TenantFilter(DustMsProperties dustMsProperties, TenantManager tenantManager) {
        this.dustMsProperties = dustMsProperties;
        this.tenantAdapterManager = tenantManager;
    }

    public IDataAdapter getAdapter() {
        if (tenantAdapterManager.isEnable()) {
            SecurityContext securityContext = SecurityContextHolder.getContext();
            Authentication authentication = securityContext.getAuthentication();
            if(authentication instanceof DustAuthentication) {
                DustUser dustUser = ((DustAuthentication) authentication).getUser();
                if (dustUser == null || StringUtils.isEmpty(dustUser.getAppId()) || StringUtils.isEmpty(dustUser.getTenantId())) {
                    throw new DustSystemException("服务公共参数异常");
                }

                return tenantAdapterManager.getAdapter(dustUser.getTenantId(), dustUser.getAppId());
            }
        }

        return tenantAdapterManager.getAdapter(null);
    }

    @Override
    public void doFilter(ServletRequest servletRequest, ServletResponse servletResponse, FilterChain filterChain) throws IOException, ServletException {
        try {
            String uri = ((HttpServletRequest)servletRequest).getRequestURI();
            if (IGNORE_URI.matcher(uri).matches()) {
                filterChain.doFilter(servletRequest, servletResponse);
                return;
            }


            if (dustMsProperties.getRepository().isAutoInitAdapter()) {
                DictGlobalConfig.Instance.setDataAdapter(getAdapter());
            }
            filterChain.doFilter(servletRequest, servletResponse);
        } finally {
            DbAdapterManager.destroy();
            LocalHolder.remove();
        }
    }

    @Override
    public void destroy() {

    }
}
